package com.yihu.wlyy.service.app.sign;

import com.yihu.wlyy.entity.doctor.profile.Doctor;
import com.yihu.wlyy.entity.organization.Hospital;
import com.yihu.wlyy.entity.patient.Patient;
import com.yihu.wlyy.repository.doctor.DoctorDao;
import com.yihu.wlyy.repository.organization.HospitalDao;
import com.yihu.wlyy.repository.patient.PatientDao;
import com.yihu.wlyy.repository.patient.SignFamilyDao;
import com.yihu.wlyy.service.BaseService;
import com.yihu.wlyy.service.app.consult.ConsultService;
import com.yihu.wlyy.service.common.SMSService;
import com.yihu.wlyy.task.PushMsgTask;
import com.yihu.wlyy.task.RemindExpensesTask;
import com.yihu.wlyy.task.RemindFocusWechatTask;
import com.yihu.wlyy.util.DateUtil;
import com.yihu.wlyy.wechat.util.WeiXinAccessTokenUtils;
import com.yihu.wlyy.wechat.util.WeiXinOpenIdUtils;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 缴费提醒服务
 * <p>
 * Created by lyr on 2016/10/24.
 */
@Service
public class PatientRemindService extends BaseService {

    @Autowired
    SignFamilyDao signFamilyDao;
    @Autowired
    DoctorDao doctorDao;
    @Autowired
    PatientDao patientDao;
    @Autowired
    HospitalDao hospitalDao;
    @Autowired
    SMSService smsService;
    @Autowired
    StringRedisTemplate redisTemplate;
    @Autowired
    WeiXinAccessTokenUtils tokenUtils;
    @Autowired
    JdbcTemplate jdbcTemplate;
    @Autowired
    ConsultService consultService;
    @Autowired
    WeiXinOpenIdUtils weiXinOpenIdUtils;
    @Autowired
    SMSService sMSService;

    /**
     * 缴费提醒
     *
     * @param doctor 医生
     * @throws Exception
     */
    public void remindPatientExpensesAll(String doctor) throws Exception {
        try {
            Doctor doc = doctorDao.findByCode(doctor);
            if (doc == null) {
                throw new Exception("doctor info can not find");
            }

            Hospital hos = hospitalDao.findByCode(doc.getHospital());
            if (hos == null) {
                throw new Exception("hospital info can not find");
            }

            RemindAll(doc, hos);
            redisTemplate.opsForValue().set("expenses:remind:doctor:" + doctor, "0");
            redisTemplate.expire("expenses:remind:doctor:" + doctor, 10, TimeUnit.MINUTES);
        } catch (Exception e) {
            redisTemplate.opsForValue().set("expenses:remind:doctor:" + doctor, "0");
            redisTemplate.expire("expenses:remind:doctor:" + doctor, 10, TimeUnit.MINUTES);
            e.printStackTrace();
        }
    }

    /**
     * 所有未缴费提醒
     *
     * @param doc
     * @param hos
     */
    public void RemindAll(Doctor doc, Hospital hos) throws Exception {
        Calendar today = Calendar.getInstance();
        today.set(Calendar.HOUR, 23);
        today.set(Calendar.MINUTE, 59);
        today.set(Calendar.SECOND, 59);
        today.set(Calendar.MILLISECOND, 999);
        boolean flag = true;
        int page = 0;
        int start = 0;
        String sql = "select p.* " +
                "     from " +
                "        wlyy_sign_family f join wlyy_patient p on f.patient = p.code " +
                "     where " +
                "        (f.doctor = ? or f.doctor_health = ?) " +
                "        and f.status > 0 " +
                "        and f.type = 2 " +
                "        and (f.expenses_status = '0' or LENGTH(trim(ifnull(f.expenses_status,''))) < 1) " +
                "     limit ?,3000";
        List<Patient> patients = new ArrayList<>();
        List<String> redisPatients = new ArrayList<>();
        Set<String> exists = redisTemplate.opsForSet().members("expenses:remind:set");

        while (flag) {
            List<Patient> result = jdbcTemplate.query(sql, new Object[]{doc.getCode(), doc.getCode(), start}, new BeanPropertyRowMapper(Patient.class));

            if (result != null && result.size() > 0) {
                patients.addAll(result);
                for (Patient p : result) {
                    if (p != null) {
                        if (exists != null && exists.contains(p.getCode())) {
                            continue;
                        }
                        redisPatients.add(p.getCode());
                    }
                }
                page = page + 1;
                start = page * 3000;
            } else {
                flag = false;
            }
        }

        if (patients.size() > 0) {
            RemindExpensesTask task = new RemindExpensesTask(patients, doc, hos,smsService);
            new Thread(task).start();
            redisTemplate.opsForSet().add("expenses:remind:set", redisPatients.toArray(new String[]{}));
            redisTemplate.expire("expenses:remind:set", today.getTimeInMillis() - Calendar.getInstance().getTimeInMillis(), TimeUnit.MILLISECONDS);
        }
    }

    /**
     * 提醒患者缴费
     *
     * @param patient 患者
     * @param doctor  医生
     * @return
     */
    public JSONObject remindPatientExpenses(String patient, String doctor) {
        try {
            Doctor doc = doctorDao.findByCode(doctor);
            if (doc == null) {
                throw new Exception("doctor info can not find");
            }

            Hospital hos = hospitalDao.findByCode(doc.getHospital());
            if (hos == null) {
                throw new Exception("hospital info can not find");
            }

            Patient p = patientDao.findByCode(patient);

            if (p == null) {
                throw new Exception("patient info can not find");
            }

            return remindPatientExpenses(p, doc, hos);
        } catch (Exception e) {
            e.printStackTrace();
            JSONObject reObj = new JSONObject();
            reObj.put("status", -1);
            reObj.put("msg", e.getMessage());
            return reObj;
        }
    }

    /**
     * 提醒患者缴费
     *
     * @param p   患者
     * @param doc 医生
     * @param hos 医院
     * @return
     */
    public JSONObject remindPatientExpenses(Patient p, Doctor doc, Hospital hos) throws Exception {
        try {
            Calendar today = Calendar.getInstance();
            today.set(Calendar.HOUR, 23);
            today.set(Calendar.MINUTE, 59);
            today.set(Calendar.SECOND, 59);
            today.set(Calendar.MILLISECOND, 59);
            JSONObject reObj = new JSONObject();
            String mDesc = "";
            boolean mFlag = false;
            boolean wFlag = false;
            boolean isMobileExist = false;
            boolean isOpenIdExist = false;

            if (StringUtils.isNotEmpty(p.getMobile())) {
                isMobileExist = true;
                JSONObject result = smsService.sendMsg(p.getMobile(), doc.getName() + "医生提醒您：为完成家庭医生签约，" +
                        "尽早为您提供家庭医生服务，请尽快到" + hos.getName() + "（地址：" + hos.getAddress() + "）缴费");
                if (result != null && result.getInt("result") == 0) {
                    mFlag = true;
                }
                if (result != null) {
                    mDesc = result.getString("description");
                }
            }
//            if (StringUtils.isNotEmpty(p.getOpenid())) {

            // 微信提醒
            JSONObject json = new JSONObject();
            json.put("first", "缴费提醒");
            json.put("toUser", p.getCode());
            json.put("represented",p.getCode());//被代理人
            json.put("name", p.getName());
            json.put("doctor", doc.getCode());
            json.put("doctorName", "提醒医生:  "+doc.getName());
            json.put("date", DateUtil.dateToStrShort(new Date()));
            json.put("content", p.getName() + "，您好！为完成家庭医生签约，尽早为您提供家庭医生服务，请尽快到" + hos.getName() + "（地址：" + hos.getAddress() + "）缴费");
            json.put("remark", "");

            //判断是否判定openId,有没有发则查找家人发送
            if(StringUtils.isNotBlank(p.getOpenid())){
                isOpenIdExist = true;
                // 添加到发送队列
                PushMsgTask.getInstance().putWxMsg(tokenUtils.getAccessToken(), 8, p.getOpenid(), p.getName(), json);
                wFlag = true;
            }
//                else{
//                    JSONObject j  = weiXinOpenIdUtils.getFamilyOpenId(p.getCode());
//                    Patient member = (Patient) j.get("member");
//                    if(StringUtils.isNotBlank(member.getOpenid())){
//                        isOpenIdExist = true;
//                        String first = (String) json.get("doctorName");
//                        json.remove("doctorName");
//                        json.put("doctorName",weiXinOpenIdUtils.getTitleMes(p,j.getInt("relation"),member.getName())+"\n"+first);
//                        PushMsgTask.getInstance().putWxMsg(tokenUtils.getAccessToken(), 8, member.getOpenid(), member.getName(), json);
//                    }
//                }
            //发送代理人
            JSONArray jsonArray = weiXinOpenIdUtils.getAgentOpenId(p.getCode(),p.getOpenid());
            if(jsonArray!=null&&jsonArray.length()>0){
                String first = (String) json.get("doctorName");
                for (int i = 0;i<jsonArray.length();i++){
                    JSONObject j  = jsonArray.getJSONObject(i);
                    Patient member = (Patient) j.get("member");
                    isOpenIdExist = true;
                    JSONObject data = json;
                    data.remove("toUser");
                    data.put("toUser",member.getCode());
                    data.remove("doctorName");
                    data.put("doctorName", weiXinOpenIdUtils.getTitleMes(p, j.getInt("relation"), p.getName()) +"\n"+first);
                    PushMsgTask.getInstance().putWxMsg(tokenUtils.getAccessToken(), 8, member.getOpenid(), p.getName(), data);
                    wFlag = true;
                }
            }


//            }
            //发送IM
            consultService.sendMucMessageBySingnType(doc.getCode(), doc.getName(), p.getCode(), doc.getName() + "医生提醒您：为完成家庭医生签约，" +
                    "尽早为您提供家庭医生服务，请尽快到" + hos.getName() + "（地址：" + hos.getAddress() + "）缴费", "1",p.getName());

            if (wFlag || mFlag) {
                reObj.put("status", 200);
                redisTemplate.opsForSet().add("expenses:remind:set", p.getCode());
                redisTemplate.expireAt("expenses:remind:set", today.getTime());
                if (wFlag && mFlag) {
                    reObj.put("msg", "提醒成功");
                } else if (mFlag && !wFlag) {
                    reObj.put("msg", "已短信提醒，居民未绑定微信，无法微信提醒");
                } else {
                    if (!isMobileExist) {
                        reObj.put("msg", "已微信提醒，居民未绑定手机，无法短信提醒");
                    } else {
                        reObj.put("msg", "已微信提醒，短信提醒失败，失败原因：" + mDesc);
                    }
                }
            } else {
                reObj.put("status", -1);

                if (!isMobileExist && !isOpenIdExist) {
                    reObj.put("msg", "居民未绑定手机及微信，无法提醒");
                } else if (isMobileExist && !isOpenIdExist) {
                    reObj.put("msg", "居民未绑定微信，且短信提醒失败，无法提醒居民。短信失败原因：" + mDesc);
                } else {
                    reObj.put("msg", "提醒失败");
                }
            }

            return reObj;
        } catch (Exception e) {
            e.printStackTrace();
            JSONObject reObj = new JSONObject();
            reObj.put("status", -1);
            reObj.put("msg", e.getMessage());
            return reObj;
        }
    }

    /**
     * 提醒患者关注微信
     *
     * @param patient 患者
     * @param doctor  医生
     * @return
     */
    public JSONObject remindPatientFocusWechat(String patient, String doctor) {
        try {
            Doctor doc = doctorDao.findByCode(doctor);
            if (doc == null) {
                throw new Exception("doctor info can not find");
            }

            Patient p = patientDao.findByCode(patient);

            if (p == null) {
                throw new Exception("patient info can not find");
            }

            Map<String, Object> map = new HashMap<>();
            map.put("code", p.getCode());
            map.put("name", p.getName());
            map.put("mobile", p.getMobile());
            return remindWechatFocus(map, doc);
        } catch (Exception e) {
            e.printStackTrace();
            JSONObject reObj = new JSONObject();
            reObj.put("status", -1);
            reObj.put("msg", e.getMessage());
            return reObj;
        }
    }

    /**
     * 提醒所有患者关注微信
     *
     * @param doctor
     */
    public void remindPatientWechatFocusAll(String doctor) throws Exception {
        try {
            Calendar today = Calendar.getInstance();
            today.set(Calendar.HOUR, 23);
            today.set(Calendar.MINUTE, 59);
            today.set(Calendar.SECOND, 59);
            today.set(Calendar.MILLISECOND, 999);
            int page = 0;
            int start = 0;
            boolean flag = true;
            Doctor doc = doctorDao.findByCode(doctor);
            if (doc == null) {
                throw new Exception("doctor info can not find");
            }
            String sql = "select p.code,p.name,p.mobile" +
                    "     from " +
                    "        wlyy_sign_family f join wlyy_patient p on f.patient = p.code" +
                    "     where " +
                    "        (f.doctor = ? or f.doctor_health = ?) " +
                    "        and f.status > 0 " +
                    "        and f.expenses_status = '1' " +
                    "        and LENGTH(trim(ifnull(p.openid,''))) < 1 limit ?,3000";
            List<Map<String, Object>> patients = new ArrayList<>();
            List<String> redisPatients = new ArrayList<>();
            Set<String> exist = redisTemplate.opsForSet().members("wechat:focus:remind:set");
            while (flag) {
                List<Map<String, Object>> result = jdbcTemplate.queryForList(sql, new Object[]{doctor, doctor, start});

                if (result != null && result.size() > 0) {
                    patients.addAll(result);
                    for (Map<String, Object> map : result) {
                        if (exist != null && exist.contains(map.get("code").toString())) {
                            continue;
                        }
                        redisPatients.add(map.get("code").toString());
                    }
                    page = page + 1;
                    start = page * 3000;
                } else {
                    flag = false;
                }
            }

            if (patients.size() > 0) {
                RemindFocusWechatTask task = new RemindFocusWechatTask(patients, doc,sMSService);
                new Thread(task).start();
                redisTemplate.opsForSet().add("wechat:focus:remind:set", redisPatients.toArray(new String[]{}));
                redisTemplate.expire("wechat:focus:remind:set", today.getTimeInMillis() - Calendar.getInstance().getTimeInMillis(), TimeUnit.MILLISECONDS);
            }

            redisTemplate.opsForValue().set("wechat:focus:remind:doctor:" + doctor, "0");
            redisTemplate.expire("wechat:focus:remind:doctor:" + doctor, 10, TimeUnit.MINUTES);
        } catch (Exception e) {
            redisTemplate.opsForValue().set("wechat:focus:remind:doctor:" + doctor, "0");
            redisTemplate.expire("wechat:focus:remind:doctor:" + doctor, 10, TimeUnit.MINUTES);
            e.printStackTrace();
        }
    }

    /**
     * 微信关注提醒
     *
     * @param p
     * @param doctor
     * @return
     */
    public JSONObject remindWechatFocus(Map<String, Object> p, Doctor doctor) {
        Calendar today = Calendar.getInstance();
        today.set(Calendar.HOUR, 23);
        today.set(Calendar.MINUTE, 59);
        today.set(Calendar.SECOND, 59);
        today.set(Calendar.MILLISECOND, 59);
        JSONObject result = new JSONObject();
        redisTemplate.opsForSet().add("wechat:focus:remind:set", p.get("code").toString());
        redisTemplate.expireAt("wechat:focus:remind:set", today.getTime());

        if (p.get("mobile") == null || StringUtils.isEmpty(p.get("mobile").toString())) {
            result.put("status", -1);
            result.put("msg", "对不起，居民未绑定手机号，无法短信提醒");
            return result;
        }

        String msg = "尊敬的" + p.get("name").toString() + "，" + doctor.getName() + "医生提醒您：" +
                "为更好地为您提供健康服务，请您关注并登录\"厦门i健康\"微信公众号，与医生进行互动。";

        JSONObject smsResult = smsService.sendMsg(p.get("mobile").toString(), msg);

        if (smsResult != null && smsResult.getInt("result") == 0) {
            result.put("status", 200);
            result.put("msg", "提醒成功");
        } else {
            result.put("status", -1);
            result.put("msg", "对不起，短信提醒失败，失败原因：" + smsResult.getString("description"));
        }

        return result;
    }
}
